@@ -49,6 +49,7 @@ class Errors(object):
         "AppNotExists": {"name": "AppNotExists", "message": "application not exists"},
         "TaskNotExists": {"name": "TaskNotExists", "message": "task not exists"},
         "TaskAlreadyFinished": {"name": "TaskAlreadyFinished", "message": "task already finished"},
+        "TaskAlreadySuccess": {"name": "TaskAlreadySuccess", "message": "task already success"},
         "TaskStillRunning": {"name": "TaskStillRunning", "message": "task still running"},
         "NodeNotExists": {"name": "NodeNotExists", "message": "node not exists"},
         "ActionNotFinished": {"name": "ActionNotFinished", "message": "action not finished"},
@@ -75,6 +76,7 @@ class Stage(object):
     running = "running"
     finished = "finished"
     stopping = "stopping"
+    recovering = "recovering"
 
 
 class Status(object):
@@ -85,6 +87,13 @@ class Status(object):
     terminate = "terminate"
 
 
+class Signal(object):
+    kill = -9
+    terminate = -15
+    stop = -15
+    cancel = -50
+
+
 class Event(object):
     events = ["fail", "success"]
     fail = "fail"
