@@ -286,6 +286,25 @@ class Scheduler(object):
         except Exception as e:
             LOG.exception(e)
 
+    def current_schedule_actions(self):
+        result = {"running": [], "pending": []}
+        try:
+            for action in self.running_actions:
+                result["running"].append({
+                    "task_id": action["task_id"],
+                    "action_name": action["name"],
+                    "update_at": str(action["update_at"]) if "update_at" in action else None,
+                })
+            for action in self.pending_actions:
+                result["pending"].append({
+                    "task_id": action["task_id"],
+                    "action_name": action["name"],
+                    "update_at": str(action["update_at"]) if "update_at" in action else None,
+                })
+        except Exception as e:
+            LOG.exception(e)
+        return result
+
     def update_finish_action(self, action_result):
         try:
             action_finish = None
@@ -303,37 +322,55 @@ class Scheduler(object):
                     Tasks.instance().update(task_id, {"result": task_info["result"]})
                     self.running_actions.remove(action_finish)
                 else: # normal task actions
+                    action_info = {"task_create_at": self.tasks[task_id]["task_info"]["create_at"]}
                     if action_result["status"] == Status.success:
-                        if "actions" in action_result["result"]:
-                            to_actions = {}
-                            for action in action_result["result"]["actions"]:
-                                self.tasks[task_id]["condition"].append(action["name"])
-                                action["task_id"] = task_id
-                                action["app_id"] = self.tasks[task_id]["app_info"]["application_id"]
-                                action["app_sha1"] = self.tasks[task_id]["app_info"]["sha1"]
-                                action["task_create_at"] = self.tasks[task_id]["task_info"]["create_at"]
-                                if "to_action" in action:
-                                    if action["to_action"] in to_actions:
-                                        to_actions[action["to_action"]].append(action["name"])
-                                    else:
-                                        to_actions[action["to_action"]] = [action["name"]]
-                                self.pending_actions.append(action)
-                            for action in self.pending_actions:
-                                if action["name"] in to_actions.keys():
-                                    action["condition"].extend(to_actions[action["name"]])
-                        self.tasks[task_id]["finished"][action_finish["name"]] = action_result
-                        self.running_actions.remove(action_finish)
-                        finish_condition = self.tasks[task_id]["condition"]
-                        current_condition = self.tasks[task_id]["finished"].keys()
-                        if len(current_condition) == len(finish_condition): # task finish & success
-                            Tasks.instance().update(task_id, {"stage": Stage.finished, "status": Status.success, "end_at": now, "result": self.tasks[task_id]["finished"]})
-                            if Event.success in self.tasks[task_id]["event_actions"]:
-                                action = self.tasks[task_id]["event_actions"][Event.success]
+                        if Stage.stopping in self.tasks[task_id] and self.tasks[task_id][Stage.stopping]: # task is stopping                                
+                            self.tasks[task_id]["finished"][action_finish["name"]] = action_result
+                            self.running_actions.remove(action_finish)
+                            Tasks.instance().update(task_id, {"stage": Stage.finished, "status": Status.kill, "end_at": now, "result": self.tasks[task_id]["finished"]})
+                            if Event.fail in self.tasks[task_id]["event_actions"]:
+                                action = self.tasks[task_id]["event_actions"][Event.fail]
                                 action["input_data"] = {"result": self.tasks[task_id]["finished"]}
+                                action["input_data"]["action_info"] = action_info
                                 self.pending_actions.append(action)
                             del self.tasks[task_id]
-                        else: # task running
-                            Tasks.instance().update(task_id, {"result": self.tasks[task_id]["finished"]})
+                        else:
+                            if "actions" in action_result["result"]: # dynamic actions
+                                to_actions = {}
+                                for action in action_result["result"]["actions"]:
+                                    if action["name"] not in self.tasks[task_id]["condition"]:
+                                        self.tasks[task_id]["condition"].append(action["name"])
+                                    action["task_id"] = task_id
+                                    action["app_id"] = self.tasks[task_id]["app_info"]["application_id"]
+                                    action["app_sha1"] = self.tasks[task_id]["app_info"]["sha1"]
+                                    action["task_create_at"] = self.tasks[task_id]["task_info"]["create_at"]
+                                    if len(action["condition"]) == 0:
+                                        action["input_data"].update(self.tasks[task_id]["task_info"]["input_data"])
+                                    action["input_data"]["action_info"] = action_info
+                                    if "to_action" in action:
+                                        if action["to_action"] in to_actions:
+                                            to_actions[action["to_action"]].append(action["name"])
+                                        else:
+                                            to_actions[action["to_action"]] = [action["name"]]
+                                    if action["name"] not in self.tasks[task_id]["finished"]:
+                                        self.pending_actions.append(action)
+                                for action in self.pending_actions:
+                                    if action["name"] in to_actions.keys():
+                                        action["condition"].extend(to_actions[action["name"]])
+                            self.tasks[task_id]["finished"][action_finish["name"]] = action_result
+                            self.running_actions.remove(action_finish)
+                            finish_condition = self.tasks[task_id]["condition"]
+                            current_condition = self.tasks[task_id]["finished"].keys()
+                            if len(current_condition) == len(finish_condition): # task finish & success
+                                Tasks.instance().update(task_id, {"stage": Stage.finished, "status": Status.success, "end_at": now, "result": self.tasks[task_id]["finished"]})
+                                if Event.success in self.tasks[task_id]["event_actions"]:
+                                    action = self.tasks[task_id]["event_actions"][Event.success]
+                                    action["input_data"] = {"result": self.tasks[task_id]["finished"]}
+                                    action["input_data"]["action_info"] = action_info
+                                    self.pending_actions.append(action)
+                                del self.tasks[task_id]
+                            else: # task running
+                                Tasks.instance().update(task_id, {"result": self.tasks[task_id]["finished"]})
                     # action failed
                     else:
                         self.tasks[task_id]["finished"][action_finish["name"]] = action_result
@@ -356,6 +393,7 @@ class Scheduler(object):
                         if Event.fail in self.tasks[task_id]["event_actions"]:
                             action = self.tasks[task_id]["event_actions"][Event.fail]
                             action["input_data"] = {"result": self.tasks[task_id]["finished"]}
+                            action["input_data"]["action_info"] = action_info
                             self.pending_actions.append(action)
                         del self.tasks[task_id]
         except Exception as e:
@@ -430,7 +468,7 @@ class Scheduler(object):
             LOG.exception(e)
 
     @gen.coroutine
-    def execute_service(self):
+    def execute_service(self): # from RAM pending to RAM running
         LOG.debug("execute_service")
         try:
             node = yield self.select_node_balanced() 
@@ -441,6 +479,9 @@ class Scheduler(object):
                 action = self.select_executable_action()
                 LOG.info("seleted action: %s", action)
                 if action:
+                    if "input_data" in action and "action_info" in action["input_data"]:
+                        action["input_data"]["action_info"]["http_host"] = http_host
+                        action["input_data"]["action_info"]["http_port"] = http_port
                     url = "http://%s:%s/action/run" % (http_host, http_port)
                     request = HTTPRequest(url = url, method = "POST", body = json.dumps(action))
                     r = yield self.async_client.fetch(request)
@@ -469,7 +510,7 @@ class Scheduler(object):
             LOG.exception(e)
 
     @gen.coroutine
-    def schedule_service(self):
+    def schedule_service(self): # from database pending / recovering to RAM pending
         LOG.debug("schedule_service")
         try:
             load_more_task = self.can_load_more_task()
@@ -487,26 +528,83 @@ class Scheduler(object):
                             app_config = json.loads(fp.read())
                             fp.close()
                             finish_condition = []
-                            for action in app_config["actions"]:
-                                action["task_id"] = task_id
-                                action["app_id"] = app_id
-                                action["app_sha1"] = app_info["sha1"]
-                                action["task_create_at"] = task_info["create_at"]
-                                if len(action["condition"]) == 0:
-                                    action["input_data"] = task_info["input_data"]
-                                finish_condition.append(action["name"])
-                                self.pending_actions.append(action)
-                            event_actions = {}
-                            if "event_actions" in app_config:
-                                for event_name in app_config["event_actions"]:
-                                    action = app_config["event_actions"][event_name]
+                            action_info = {"task_create_at": task_info["create_at"]}
+                            if task_info["stage"] == Stage.pending: # load pending task
+                                for action in app_config["actions"]:
                                     action["task_id"] = task_id
                                     action["app_id"] = app_id
                                     action["app_sha1"] = app_info["sha1"]
                                     action["task_create_at"] = task_info["create_at"]
-                                event_actions = app_config["event_actions"]
-                            self.tasks[task_id] = {"task_info": task_info, "condition": finish_condition, "app_info": app_info, "finished": {}, "event_actions": event_actions}
-                            Tasks.instance().update(task_id, {"stage": Stage.running, "start_at": datetime.datetime.now()})
+                                    action["input_data"] = {}
+                                    if len(action["condition"]) == 0:
+                                        action["input_data"] = task_info["input_data"]
+                                    action["input_data"]["action_info"] = action_info
+                                    finish_condition.append(action["name"])
+                                    self.pending_actions.append(action)
+                                event_actions = {}
+                                if "event_actions" in app_config:
+                                    for event_name in app_config["event_actions"]:
+                                        action = app_config["event_actions"][event_name]
+                                        action["task_id"] = task_id
+                                        action["app_id"] = app_id
+                                        action["app_sha1"] = app_info["sha1"]
+                                        action["task_create_at"] = task_info["create_at"]
+                                    event_actions = app_config["event_actions"]
+                                self.tasks[task_id] = {"task_info": task_info, "condition": finish_condition, "app_info": app_info, "finished": {}, "event_actions": event_actions}
+                                Tasks.instance().update(task_id, {"stage": Stage.running, "start_at": datetime.datetime.now()})
+                            elif task_info["stage"] == Stage.recovering: # load recovering task
+                                actions_tmp = {}
+                                for action in app_config["actions"]: # load configuration actions
+                                    action["task_id"] = task_id
+                                    action["app_id"] = app_id
+                                    action["app_sha1"] = app_info["sha1"]
+                                    action["task_create_at"] = task_info["create_at"]
+                                    action["input_data"] = {}
+                                    if len(action["condition"]) == 0:
+                                        action["input_data"] = task_info["input_data"]
+                                    action["input_data"]["action_info"] = action_info
+                                    finish_condition.append(action["name"])
+                                    actions_tmp[action["name"]] = action
+                                event_actions = {}
+                                if "event_actions" in app_config:
+                                    for event_name in app_config["event_actions"]:
+                                        action = app_config["event_actions"][event_name]
+                                        action["task_id"] = task_id
+                                        action["app_id"] = app_id
+                                        action["app_sha1"] = app_info["sha1"]
+                                        action["task_create_at"] = task_info["create_at"]
+                                    event_actions = app_config["event_actions"]
+                                task_result = {}
+                                actions_dynamic = []
+                                for action_name in task_info["result"]: # load result actions
+                                    action = task_info["result"][action_name]
+                                    if action["stage"] == Stage.finished and action["status"] == Status.success:
+                                        if action_name in actions_tmp: # remove success action
+                                            del actions_tmp[action_name]
+                                        task_result[action_name] = action
+                                        if "actions" in action["result"] and action["result"]["actions"]:
+                                            actions_dynamic.extend(action["result"]["actions"])
+                                for action in actions_dynamic: # load dynamic actions
+                                    to_action = action["to_action"] if "to_action" in action and action["to_action"] else None
+                                    if to_action and to_action in actions_tmp:
+                                        actions_tmp[to_action]["condition"].append(action["name"])
+                                    if action["name"] not in task_result: # failed dynamic action, append into actions_tmp
+                                        action["task_id"] = task_id
+                                        action["app_id"] = app_id
+                                        action["app_sha1"] = app_info["sha1"]
+                                        action["task_create_at"] = task_info["create_at"]
+                                        action["input_data"]["action_info"] = action_info
+                                        if "signal" in action:
+                                            del action["signal"]
+                                        actions_tmp[action["name"]] = action
+                                    finish_condition.append(action["name"])
+                                for name in actions_tmp:
+                                    self.pending_actions.append(actions_tmp[name])
+                                self.tasks[task_id] = {"task_info": task_info, "condition": finish_condition, "app_info": app_info, "finished": task_result, "event_actions": event_actions}
+                                LOG.debug("recover task, task_id: %s, condition: %s, finished: %s", task_id, finish_condition, task_result)
+                                Tasks.instance().update(task_id, {"stage": Stage.running, "start_at": datetime.datetime.now()})
+                            else:
+                                LOG.error("unknown task stage value: %s", task_info["stage"])
                         else:
                             LOG.error("Scheduler app config file[%s] not exists", app_config_path)
                     elif app_info is None:
