@@ -83,9 +83,13 @@ parser_task_rerun = subparsers_task.add_parser("rerun", help = "rerun task")
 parser_task_rerun.add_argument("-t", "--task_id", required = True, help = "task id", default = "")
 parser_task_rerun.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
 
+parser_task_recover = subparsers_task.add_parser("recover", help = "recover task")
+parser_task_recover.add_argument("-t", "--task_id", required = True, help = "task id", default = "")
+parser_task_recover.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
 parser_task_stop = subparsers_task.add_parser("stop", help = "stop task")
 parser_task_stop.add_argument("-t", "--task_id", required = True, help = "task id", default = "")
-parser_task_stop.add_argument("-g", "--signal", help = "stop task's signal: -9 or -15", type = int, default = -15)
+parser_task_stop.add_argument("-g", "--signal", help = "stop task's signal: -9 or -15, default is -9", type = int, default = -9)
 parser_task_stop.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
 
 # operate with cluster
@@ -312,11 +316,11 @@ def main():
                             values["description"] = args.description
                         if executable:
                             if files:
-                                r = requests.put(url, files = files, data = values)
+                                r = requests.post(url, files = files, data = values)
                             else:
                                 for key in values:
                                     values[key] = (None, values[key])
-                                r = requests.put(url, files = values)
+                                r = requests.post(url, files = values)
                             if r.status_code == 200:
                                 data = r.json()
                                 if raw:
@@ -465,6 +469,26 @@ def main():
                             print(e)
                     else:
                         parser.print_help()
+                elif operation == "recover":
+                    if args.task_id:
+                        try:
+                            data = {"task_id": args.task_id}
+                            r = requests.put(url, json = data)
+                            if r.status_code == 200:
+                                data = r.json()
+                                if raw:
+                                    print(json.dumps(data, indent = 4, sort_keys = True))
+                                else:
+                                    print_table_result(
+                                        [data],
+                                        ["result", "message"]
+                                    )
+                            else:
+                                print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                        except Exception as e:
+                            print(e)
+                    else:
+                        parser.print_help()
                 elif operation == "stop":
                     if args.task_id and args.signal:
                         try:
